/********************************************************
Workspace: IAR+SPL
Chip type: STM8S003K3
Project  : thermofan controller
Version  : 2
Date     : Dec 2017
*********************************************************/

#include "stm8s.h"
#include "iostm003.c"
#include "bitset.c"
#include "stdbool.h"
#include "defines.c"
#include "periph_init.c"
#include "kwork.c"
#include "eeprom_routines.c"
#include "apow_work.c"
#include "ledwork.c"

volatile u16 tim0,tim1,tim2,tim3,tim4;
volatile bool iwdg_t4,iwdg_t2;

u8 bta_angle;
u8 z01_angle;

extern "C"
{
 INTERRUPT_HANDLER(TIM1_CAPCOM, 12)
 {
  u16 tmp;

  CLK->CKDIVR = 0;

  if(TIM1->SR1&TIM1_SR1_CC1IF && TIM1->IER&TIM1_IER_CC1IE)
  {
   tmp = TIM1->CCR1H<<8;
   tmp|= TIM1->CCR1L;

   tmp+=pw;
   TIM1->CCR3H = TIM1->CCR2H = TIM1->CCR1H = tmp>>8;
   TIM1->CCR3L = TIM1->CCR2L = TIM1->CCR1L = (u8)tmp;

   TIM1->IER  &= ~TIM1_IER_CC1IE;
   TIM1->SR1  &= ~TIM1_SR1_CC1IF;
  }

  if(TIM1->SR1&TIM1_SR1_CC4IF && TIM1->IER&TIM1_IER_CC4IE)
  {
   tmp = TIM1->CCR4H<<8;
   tmp|= TIM1->CCR4L;

   tmp+=pw;
   TIM1->CCR4H = tmp>>8;
   TIM1->CCR4L = (u8)tmp;
   TIM1->IER  &= ~TIM1_IER_CC4IE;
   TIM1->SR1  &= ~TIM1_SR1_CC4IF;
  }

  CLK->CKDIVR = mcu_div;
 }

 INTERRUPT_HANDLER(TIM2_CAPCOM, 14)
 {
  CLK->CKDIVR  = 0;

  u16 tmp, atmp;

  tmp = TIM2->CCR2H<<8;
  tmp|= TIM2->CCR2L;

  if(tmp<(u16)((u32)mcu_freq/200/16))goto exit_cc2;

  TIM2->EGR = TIM2_EGR_UG;
  TIM1->EGR = TIM1_EGR_UG;
  TIM2->CCER1^= TIM2_CCER1_CC2P;

  TIM1->CCMR4 =TIM1->CCMR3 =TIM1->CCMR2 =TIM1->CCMR1 = force_active;

  if(bta_angle<0xff)
  {
   TIM1->CCMR3 =TIM1->CCMR2 =TIM1->CCMR1 = toggle_on_match;

   atmp = (u32)bta_angle*tmp>>8;
   TIM1->CCR3H = TIM1->CCR2H = TIM1->CCR1H = atmp>>8;
   TIM1->CCR3L = TIM1->CCR2L = TIM1->CCR1L = (u8)atmp;
   TIM1->IER  |= TIM1_IER_CC1IE;

   TIM2->CCR1H = TIM1->CCR1H;
   TIM2->CCR1L = TIM1->CCR1L;
  }
  else
  {
   TIM1->CCMR3 =TIM1->CCMR2 =TIM1->CCMR1 = force_active;
  }

  if(z01_angle<0xff)
  {
   TIM1->CCMR4 = toggle_on_match;

   atmp = (u32)z01_angle*tmp>>8;
   TIM1->CCR4H = atmp>>8;
   TIM1->CCR4L = (u8)atmp;
   TIM1->IER  |= TIM1_IER_CC4IE;
  }
  else
  {
   TIM1->CCMR4 = force_active;
  }

  TIM1->SR1   = 0;

  PF_ODR ^= 0xff;

  exit_cc2:
  iwdg_t2 = 1;
  TIM2->SR1 = 0;
  CLK->CKDIVR  = mcu_div;
 }

 INTERRUPT_HANDLER(TIM4_OVF, 23)   //1000Hz
 {
  CLK->CKDIVR = 0;

  static u8 cnt;
  if(cnt)cnt--;
  else
  {
   cnt = 9;
   PC_ODR_ODR7^=1;
  }


  if(tim0)tim0--;
  if(tim1)tim1--;
  if(tim2)tim2--;
  if(tim3)tim3--;
  if(tim4)tim4--;

  scan_keys();

  iwdg_t4=1;
  TIM4->SR1 = 0;
  CLK->CKDIVR  = mcu_div;
 }
}

void devcon(void);

void main()
{
 cfg.gpio();
 cfg.clk();
 cfg.t1();
 cfg.t2();
 cfg.t4();
 cfg.ee();

 calc_pw();

 ITC_SetSoftwarePriority(ITC_IRQ_TIM1_CAPCOM, ITC_PRIORITYLEVEL_1);
 ITC_SetSoftwarePriority(ITC_IRQ_TIM2_CAPCOM, ITC_PRIORITYLEVEL_1);
 ITC_SetSoftwarePriority(ITC_IRQ_TIM4_OVF, ITC_PRIORITYLEVEL_0);

 cfg.iwdg();

 asm("rim");

 endless
 {
  devcon();

  if(iwdg_t4 && iwdg_t2){iwdg_t4=iwdg_t2=0; IWDG_ReloadCounter();}
 }
}

void devcon()
{
 static bool itr;
 static enum {start,work,sdn}dev_st = start;
 static bool btn_trg;

 if(btn[0]==rls && btn[1]==rls)
 {
  btn_trg=0;
 }

 switch (dev_st)
 {
  case start:
  {
   if(!itr)
   {
    itr=1;
    led = 1;
    tim0=500;
   }

   if(!tim0)
   {
    if(read_ee_u8(eep.onoff, sizeof(eep.onoff))==1)dev_st=work;
    else dev_st=sdn;
   }

   if(dev_st!=start){itr=0;}
  }
  break;

  case work:
  {
   static u8 dv_tmp[2],buf[2];

   if(!itr)
   {
    itr=1;
    dv_tmp[1] = read_ee_u8(eep.airflow, sizeof(eep.airflow));
    z01_angle = set_pow (dv_tmp[1]);
    dv_tmp[0] = read_ee_u8(eep.heatpow, sizeof(eep.heatpow));
    bta_angle = set_pow (dv_tmp[0]);
   }


   static bool pb_trg[2];

   for(u8 i=0;i<2;i++)
   {
    if(btn[i]==prs)
    {
     pb_trg[i]=1;
    }
    else if(btn[i]==hold)
    {
     pb_trg[i]=0;

     if(btn[1]==rls || btn[0]==rls)
     if(!tim0)
     {
      tim0=250;

      if(!i)
      {
       if(dv_tmp[0]<dv_tmp[1])dv_tmp[i]++;
       else dv_tmp[i]=0;
      }
      else
      {
       if(dv_tmp[i]<9)dv_tmp[i]++;
       else dv_tmp[i]=3;
      }
     }
    }
    else
    {
     if(pb_trg[i])
     {
      if(!i)
      {
       if(dv_tmp[0]<dv_tmp[1])dv_tmp[i]++;
       else dv_tmp[i]=0;
      }
      else
      {
       if(dv_tmp[i]<9)dv_tmp[i]++;
       else dv_tmp[i]=3;
      }

      pb_trg[i]=0;
     }
    }
   }

   if(buf[1]!=dv_tmp[1])
   {
    buf[1]=dv_tmp[1];
    z01_angle = set_pow (dv_tmp[1]);

    if(buf[1]>=buf[0]) bta_angle = set_pow (buf[0]);
    else               bta_angle = set_pow (buf[1]);
   }
   if(buf[0]!=dv_tmp[0])
   {
    buf[0]=dv_tmp[0];
    bta_angle = set_pow (dv_tmp[0]);
   }

   if(buf[1]<buf[0])led_blink(buf[1], tim1);
   else             led_blink(buf[0], tim1);

   if(!btn_trg && btn[0]==hold && btn[1]==hold)
   {
    btn_trg=1;
    dev_st = sdn;
   }


   if(dev_st!=work)
   {
    itr=0;
    write_ee_u8(dv_tmp[1], eep.airflow, sizeof(eep.airflow));
    write_ee_u8(dv_tmp[0], eep.heatpow, sizeof(eep.heatpow));
   }
  }
  break;

  case sdn:
  {
   if(!itr)
   {
    itr=1;
    bta_angle = set_pow (0);
    z01_angle = set_pow (9);

    if(read_ee_u8(eep.onoff, sizeof(eep.onoff))==1)
    {
     tim0 = 0xffff/2;
     write_ee_u8(0, eep.onoff, sizeof(eep.onoff));
    }
    else tim0=0;
   }

   if(!tim0)
   {
    z01_angle = set_pow (0);
    led = 0;
   }
   else
   {
    if(!tim3)
    {
     tim3=1000;
     led^=1;
    }
   }

   if(!btn_trg)
   if(btn[0]==hold && btn[1]==hold)
   {
    btn_trg=1;
    dev_st = start;
    write_ee_u8(1, eep.onoff, sizeof(eep.onoff));
   }

   if(dev_st!=sdn){itr=0;}
  }
  break;
 }
}